#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json



HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}


def get_dev_asset_details(ipaddress, auth, url):
    """Takes in ipaddress as input to fetch device assett details from HP IMC RESTFUL API

    :param ipaddress: IP address of the device you wish to gather the asset details

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: object of type list containing the device asset details, with each asset contained in a dictionary

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.netassets import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_dev_asset_details('10.101.0.1', auth.creds, auth.url)
Out[46]:
[{'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=1',
  'assetNumber': '',
  'boardNum': 'FHK1119F1DX',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '0',
  'desc': '2811 chassis',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': 'System Bootstrap, Version 12.4(13r)T11, RELEASE SOFTWARE (fc1)',
  'hardVersion': 'V04 ',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': 'CISCO2811',
  'name': '2811 chassis',
  'phyClass': '3',
  'phyIndex': '1',
  'physicalFlag': '0',
  'relPos': '-1',
  'remark': '',
  'serialNum': 'FHK1119F1DX',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '15.1(4)M, RELEASE SOFTWARE (fc1)',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.3.436'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=14',
  'assetNumber': '',
  'boardNum': 'FOC11063NZ4',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '13',
  'desc': '2nd generation two port FXO voice interface daughtercard',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': 'V01 ',
  'isFRU': '1',
  'mfgName': 'Cisco',
  'model': 'VIC2-2FXO',
  'name': '2nd generation two port FXO voice interface daughtercard on Slot 0 SubSlot 2',
  'phyClass': '9',
  'phyIndex': '14',
  'physicalFlag': '2',
  'relPos': '0',
  'remark': '',
  'serialNum': 'FOC11063NZ4',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.9.3.114'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=30',
  'assetNumber': '',
  'boardNum': 'FOC11163P04',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '29',
  'desc': '40GB IDE Disc Daughter Card',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': '',
  'isFRU': '1',
  'mfgName': 'Cisco',
  'model': ' ',
  'name': '40GB IDE Disc Daughter Card on Slot 1 SubSlot 0',
  'phyClass': '9',
  'phyIndex': '30',
  'physicalFlag': '2',
  'relPos': '0',
  'remark': '',
  'serialNum': 'FOC11163P04',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.9.15.25'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=25',
  'assetNumber': '',
  'boardNum': '',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '3',
  'desc': 'AIM Container Slot 0',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': '',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': '',
  'name': 'AIM Container Slot 0',
  'phyClass': '5',
  'phyIndex': '25',
  'physicalFlag': '0',
  'relPos': '6',
  'remark': '',
  'serialNum': '',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.5.2'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=26',
  'assetNumber': '',
  'boardNum': '',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '3',
  'desc': 'AIM Container Slot 1',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': '',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': '',
  'name': 'AIM Container Slot 1',
  'phyClass': '5',
  'phyIndex': '26',
  'physicalFlag': '0',
  'relPos': '7',
  'remark': '',
  'serialNum': '',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.5.2'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=2',
  'assetNumber': '',
  'boardNum': '',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '1',
  'desc': 'C2811 Chassis Slot',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': '',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': '',
  'name': 'C2811 Chassis Slot 0',
  'phyClass': '5',
  'phyIndex': '2',
  'physicalFlag': '0',
  'relPos': '0',
  'remark': '',
  'serialNum': '',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.5.1'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=27',
  'assetNumber': '',
  'boardNum': '',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '1',
  'desc': 'C2811 Chassis Slot',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': '',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': '',
  'name': 'C2811 Chassis Slot 1',
  'phyClass': '5',
  'phyIndex': '27',
  'physicalFlag': '0',
  'relPos': '1',
  'remark': '',
  'serialNum': '',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.5.1'},
 {'alias': '',
  'asset': 'http://kontrolissues.thruhere.net:8086/imcrs/netasset/asset/detail?devId=15&phyIndex=3',
  'assetNumber': '',
  'boardNum': 'FOC11152G90',
  'bom': '',
  'buildInfo': '',
  'cleiCode': '',
  'containedIn': '2',
  'desc': 'c2811 Motherboard with 2FE and integrated VPN',
  'devId': '15',
  'deviceIp': '10.101.0.1',
  'deviceName': 'router.lab.local',
  'firmwareVersion': '',
  'hardVersion': 'V04 ',
  'isFRU': '2',
  'mfgName': 'Cisco',
  'model': '',
  'name': 'c2811 Motherboard with 2FE and integrated VPN on Slot 0',
  'phyClass': '9',
  'phyIndex': '3',
  'physicalFlag': '0',
  'relPos': '0',
  'remark': '',
  'serialNum': 'FOC11152G90',
  'serverDate': '2016-01-26T15:20:40-05:00',
  'softVersion': '15.1(4)M',
  'vendorType': '1.3.6.1.4.1.9.12.3.1.9.3.129'}]



    """
    get_dev_asset_url = "/imcrs/netasset/asset?assetDevice.ip=" + str(ipaddress)
    f_url = url + get_dev_asset_url
    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=HEADERS)
    # r.status_code
    try:
        if r.status_code == 200:
            dev_asset_info = (json.loads(r.text))
            if len(dev_asset_info) > 0:
                dev_asset_info = dev_asset_info['netAsset']
            if type(dev_asset_info) == dict:
                dev_asset_info = [dev_asset_info]
            if type(dev_asset_info) == list:
                dev_asset_info[:] = [dev for dev in dev_asset_info if dev.get('deviceIp') == ipaddress]
            return dev_asset_info
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_dev_asset_details: An Error has occured'

def get_dev_asset_details_all(auth, url):
    """Takes no input to fetch device assett details from HP IMC RESTFUL API

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionatires containing the device asset details

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.netassets import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")
    #Output in test system too long to include. Provided for example only
    #>>> get_dev_asset_details_all( auth.creds, auth.url)

    """
    # checks to see if the imc credentials are already available
    get_dev_asset_details_all_url = "/imcrs/netasset/asset?start=0&size=15000"
    f_url = url + get_dev_asset_details_all_url
    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=HEADERS)
    # r.status_code
    try:
        if r.status_code == 200:
            dev_asset_info = (json.loads(r.text))['netAsset']
            return dev_asset_info
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_dev_asset_details: An Error has occured'






#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json
from pyhpeimc.plat.device import *
from pyhpeimc.plat.vlanm import *
from pyhpeimc.plat.termaccess import *
from pyhpeimc.plat.netassets import *
from pyhpeimc.plat.alarms import *
from requests.auth import HTTPDigestAuth


headers = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

# IMC Device Class

class IMCDev:
    """
    imc_dev class takes in the ip_address which is used as the primary key to gather the following attributes
         for a device which as been previously discovered in the HP IMC Network Management platform.
         Each instance of this class should have the following attributes

         ip: The IP address used to manage the device in HP IMC
         description: returns the description of the device as discovered in HP IMC
         location: returns the location of the device as discovered in HP IMC
         contact: returns the contact of the device as discovered in HP IMC
         type: returns the type of the device as discovered in HP IMC
         name: returns the name of the device as discovered in HP IMC
         status: returns the current alarm status as discovered in HP IMC
         devid: returns the current devid used to internally identify the device as discovered in HP IMC
         interfacelist: returns the current list of interfaces for the device as discovered in HP IMC
         numinterface: returns a count of the number of interfaces in the interfacelist attribute
         vlans: returns the current vlans existing in the device as discovered in HP IMC. Device must be supported in the
                HP IMC Platform VLAN manager module.
         accessinterfaces: returns the device interfaces configured as access interfaces. Device must be supported in the
                HP IMC Platform VLAN manager module.
         trunkinterfaces: returns the device interfaces configured as trunk interfaces. Device must be supported in the
                HP IMC Platform VLAN manager module.
         alarm: returns the current unrecovered alarms as known by HP IMC.
         num alarms: returns a count of the number of alarms as returned by the alarm attribute
         serial: returns the network assets, including serial numbers for the device as discovered by HP IMC. The device
                must support the ENTITY MIB ( rfc 4133 ) for this value to be returned.
         runconfig: returns the most recent running configuration for the device as known by HP IMC. The device must be
                be supported in the HP IMC platform ICC module.
         startconfig: returns the most recent startup configuration for the device as known by HP IMC. The device must be
                be supported in the HP IMC platform ICC module.
         ipmacarp: returns the current device maciparp table as discovered by HP IMC.

         The imc_dev class supports the following methods which can be called upon an instance of this class

         addvlan: This method executes the addvlan function on the specific instance of the imc_dev object. Devices must
                  supported in the HP IMC Platform VLAN Manager module.

         """

    def __init__(self, ip_address, auth, url):
        self.ip = get_dev_details(ip_address, auth, url)['ip']
        self.description = get_dev_details(ip_address, auth, url)['sysDescription']
        self.location = get_dev_details(ip_address, auth, url)['location']
        self.contact = get_dev_details(ip_address, auth, url)['contact']
        self.type = get_dev_details(ip_address, auth, url)['typeName']
        self.name = get_dev_details(ip_address, auth, url)['sysName']
        self.status = get_dev_details(ip_address, auth, url)['statusDesc']
        self.devid = get_dev_details(ip_address, auth, url)['id']
        self.interfacelist = get_dev_interface(self.devid, auth, url)
        self.numinterface = len(get_dev_interface(self.devid, auth, url))
        self.vlans = get_dev_vlans(self.devid, auth, url)
        self.accessinterfaces = get_device_access_interfaces(self.devid, auth, url)
        self.trunkinterfaces = get_trunk_interfaces(self.devid, auth, url)
        self.alarm = get_dev_alarms(self.devid, auth, url)
        self.numalarm = len(get_dev_alarms(self.devid, auth, url))
        self.assets = get_dev_asset_details(self.ip, auth, url)
        self.serials = [({'name' : asset['name'], 'serialNum': asset['serialNum']}) for asset in self.assets]
        self.runconfig = get_dev_run_config(self.devid, auth, url)
        self.startconfig = get_dev_start_config(self.devid, auth, url)
        self.ipmacarp = get_ip_mac_arp_list(self.devid, auth, url)
        self.auth = auth
        self.url = url

    def getvlans(self):
        self.vlans = get_dev_vlans(self.devid, self.auth, self.url)

    def addvlan(self, vlanid, vlan_name,auth, url):
        create_dev_vlan(self.devid, vlanid, vlan_name, auth, url)

    def delvlan(self, vlanid, auth, url):
        delete_dev_vlans(self.devid, vlanid, auth, url)

    def getipmacarp(self):
        self.ipmacarp = get_ip_mac_arp_list(self.devid, auth, url)


class IMCInterface:
    def __init__(self, ip_address, ifIndex, auth, url):
        self.ip = get_dev_details(ip_address,auth, url)['ip']
        self.devid = get_dev_details(ip_address,auth, url)['id']
        self.ifIndex = get_interface_details(self.devid, ifIndex,auth, url)['ifIndex']
        self.macaddress = get_interface_details(self.devid, ifIndex,auth, url)['phyAddress']
        self.status = get_interface_details(self.devid, ifIndex,auth, url)['statusDesc']
        self.adminstatus = get_interface_details(self.devid, ifIndex,auth, url)['adminStatusDesc']
        self.name = get_interface_details(self.devid, ifIndex,auth, url)['ifDescription']
        self.description = get_interface_details(self.devid, ifIndex,auth, url)['ifAlias']
        self.mtu = get_interface_details(self.devid, ifIndex,auth, url)['mtu']
        self.speed = get_interface_details(self.devid, ifIndex,auth, url)['ifspeed']
        self.accessinterfaces = get_device_access_interfaces(self.devid,auth, url)
        self.pvid = get_access_interface_vlan(self.ifIndex, self.accessinterfaces,auth, url)
        self.auth = auth
        self.url = url


class IPScope:
    def __init__(self, netaddr, auth, url):
        self.id = get_scope_id(netaddr, auth, url)
        self.details = get_ip_scope_detail(auth, url, self.id)
        self.hosts = get_ip_scope_hosts(auth, url, self.id)
        self.auth = auth
        self.url = url
        self.netaddr = ipaddress.ip_network(netaddr)
        self.startip = get_ip_scope_detail(auth, url, self.id)['startIp']
        self.endip = get_ip_scope_detail(auth, url, self.id)['endIp']
        if 'assignedIpScope' in self.details:
            self.child = get_ip_scope_detail(auth, url, self.id)['assignedIpScope']

    def allocateIp(self, ipaddress, name, description):
        add_scope_ip(ipaddress, name, description, self.id, self.auth, self.url)

    def deallocateIp(self, hostid):
        remove_scope_ip( hostid, self.auth, self.url)

    def gethosts(self):
        self.hosts = get_ip_scope_hosts(self.auth, self.url, self.id)


    def nextfreeip(self):
        allocated_ips = [ ipaddress.ip_address(host['ip']) for host in self.hosts]
        for ip in self.netaddr:
            if str(ip).split('.')[-1] == '0':
                continue
            if ip not in allocated_ips:
                return ip

    def addchild(self, startIp, endIp, name, description):
        add_child_ip_scope(self.auth, self.url,startIp, endIp, name, description,self.id)









#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json
from pyhpeimc.auth import IMCAuth

HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

#auth = IMCAuth('http://','10.101.0.201','8080', 'admin','admin')

headers = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

def get_dev_alarms(devId, auth, url):
    """
    function takes the devId of a specific device and issues a RESTFUL call to get the current alarms for the target
    device.

    :param devId: int or str value of the target device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries containing the alarms for this device

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_dev_alarms('10', auth.creds.auth.url)
    [{'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/23 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160612',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467765645',
  'faultTimeDesc': '2016-07-05 20:40:45',
  'holdInfo': '',
  'id': '160612',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=85;Interface Description=GigabitEthernet2/0/23;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467765645',
  'recTimeDesc': '2016-07-05 20:40:45',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/23 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160539',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467747889',
  'faultTimeDesc': '2016-07-05 15:44:49',
  'holdInfo': '',
  'id': '160539',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=85;Interface Description=GigabitEthernet2/0/23;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467765645',
  'recTimeDesc': '2016-07-05 20:40:45',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/20 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160524',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467747461',
  'faultTimeDesc': '2016-07-05 15:37:41',
  'holdInfo': '',
  'id': '160524',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=82;Interface Description=GigabitEthernet2/0/20;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160496',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467745458',
  'faultTimeDesc': '2016-07-05 15:04:18',
  'holdInfo': '',
  'id': '160496',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/20 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160487',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742807',
  'faultTimeDesc': '2016-07-05 14:20:07',
  'holdInfo': '',
  'id': '160487',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=82;Interface Description=GigabitEthernet2/0/20;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467742807',
  'recTimeDesc': '2016-07-05 14:20:07',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/19 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160485',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742775',
  'faultTimeDesc': '2016-07-05 14:19:35',
  'holdInfo': '',
  'id': '160485',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=81;Interface Description=GigabitEthernet2/0/19;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467854218',
  'recTimeDesc': '2016-07-06 21:16:58',
  'recUserName': 'admin',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet2/0/19 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160484',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467742742',
  'faultTimeDesc': '2016-07-05 14:19:02',
  'holdInfo': '',
  'id': '160484',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=81;Interface Description=GigabitEthernet2/0/19;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467742742',
  'recTimeDesc': '2016-07-05 14:19:02',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160447',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736157',
  'faultTimeDesc': '2016-07-05 12:29:17',
  'holdInfo': '',
  'id': '160447',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736157',
  'recTimeDesc': '2016-07-05 12:29:17',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.2.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is DOWN.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160446',
  'alarmLevel': '2',
  'alarmLevelDesc': 'Major',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736125',
  'faultTimeDesc': '2016-07-05 12:28:45',
  'holdInfo': '',
  'id': '160446',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=2',
  'parentId': '160438',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736157',
  'recTimeDesc': '2016-07-05 12:29:17',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'},
 {'OID': '1.3.6.1.6.3.1.1.5.3.0',
  'ackStatus': '1',
  'ackStatusDesc': 'Acknowledged',
  'ackTime': '1467854226',
  'ackTimeDesc': '2016-07-06 21:17:06',
  'ackUserName': 'admin',
  'alarmCategory': '3',
  'alarmCategoryDesc': 'Interface/Link Status Alarm',
  'alarmDesc': 'The interface GigabitEthernet1/0/21 is UP.',
  'alarmDetail': 'http://kontrolissues.thruhere.net:8086/imcrs/fault/alarm/160444',
  'alarmLevel': '5',
  'alarmLevelDesc': 'Info',
  'deviceId': '10',
  'deviceIp': '192.168.1.221',
  'deviceName': 'HP_5500EI',
  'faultTime': '1467736092',
  'faultTimeDesc': '2016-07-05 12:28:12',
  'holdInfo': '',
  'id': '160444',
  'originalType': '1',
  'originalTypeDesc': 'Trap',
  'paras': '*Interface Index=21;Interface Description=GigabitEthernet1/0/21;Interface Admin Status=1;Interface Operate Status=1',
  'parentId': '0',
  'recStatus': '1',
  'recStatusDesc': 'Recovered',
  'recTime': '1467736092',
  'recTimeDesc': '2016-07-05 12:28:12',
  'recUserName': '$SYSTEM',
  'remark': '',
  'somState': '0'}]
    """
    # checks to see if the imc credentials are already available
    get_dev_alarm_url = "/imcrs/fault/alarm?operatorName=admin&deviceId=" + \
                        str(devId) + "&desc=false"
    f_url = url + get_dev_alarm_url
    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=headers)
    try:
        if r.status_code == 200:
            dev_alarm = (json.loads(r.text))
            if 'alarm' in dev_alarm:
                return dev_alarm['alarm']
            else:
                return "Device has no alarms"
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_dev_alarms: An Error has occured'



#!/usr/bin/env python3
# author: @netmanchris

# This section imports required libraries
import requests
import json



HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}


"""
This section deals with HPE IMC Custom View functions
"""

def get_custom_views(auth, url,name=None,headers=HEADERS):
    """
    function requires no input and returns a list of dictionaries of custom views from an HPE IMC. Optional name
    argument will return only the specified view.
    :param name: str containing the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param name: (optional) str of name of specific custom view

    :return: list of dictionaties containing attributes of the custom views

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")
    #Get All views
    >>> get_custo_views(auth.creds, auth.url)
    [{'name': 'My Network View',
  'runStatus': '5',
  'statusDesc': 'Critical',
  'statusImgSrc': 'res/icon_custom_view_critical_16x16.png',
  'symbolId': '11'},
 {'name': 'SimpleView',
  'runStatus': '4',
  'statusDesc': 'Major',
  'statusImgSrc': 'res/icon_custom_view_major_16x16.png',
  'symbolId': '2700'},
 {'name': 'View1',
  'runStatus': '-1',
  'statusDesc': 'Unmanaged',
  'statusImgSrc': 'res/icon_custom_view_unmanaged_16x16.png',
  'symbolId': '1619'}]
    #Get details of a single view
    >>>  get_custom_views(auth.creds, auth.url, name='My Network View')
  [{'name': 'My Network View',
  'runStatus': '5',
  'statusDesc': 'Critical',
  'statusImgSrc': 'res/icon_custom_view_critical_16x16.png',
  'symbolId': '11'}]
    """
    if name is None:
        get_custom_view_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&desc=false&total=false'
    elif name is not None:
        get_custom_view_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&name='+name+'&desc=false&total=false'
    f_url = url + get_custom_view_url
    r = requests.get(f_url, auth=auth, headers=headers)
    try:
        if r.status_code == 200:
            custom_view_list = (json.loads(r.text))["customView"]
            if type(custom_view_list) == dict:
                custom_view_list = [custom_view_list]
                return custom_view_list
            else:
                return custom_view_list
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_custom_views: An Error has occured'


def create_custom_views(auth, url,name=None, upperview=None):
    """
    function takes no input and issues a RESTFUL call to get a list of custom views from HPE IMC. Optioanl Name input
    will return only the specified view.

    :param name: string containg the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str of creation results ( "view " + name + "created successfully"

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    #Create L1 custom view
    >>> create_custom_views(auth.creds, auth.url, name='L1 View')
    'View Test View created successfully'

    #Create Nested custome view
    >>> create_custom_views(auth.creds, auth.url, name='L2 View', upperview='L1 View')
    'View L2 View created successfully'



    """
    create_custom_views_url = '/imcrs/plat/res/view/custom?resPrivilegeFilter=false&desc=false&total=false'
    f_url = url + create_custom_views_url
    if upperview is None:
        payload = '''{ "name": "''' + name + '''",
         "upLevelSymbolId" : ""}'''
        #print (payload)
    else:
        parentviewid = get_custom_views(auth, url, upperview)[0]['symbolId']
        payload = '''{ "name": "'''+name+ '''",
        "upLevelSymbolId" : "'''+str(parentviewid)+'''"}'''
        #print (payload)
    r = requests.post(f_url, data = payload, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 201:
            return 'View ' + name +' created successfully'
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_custom_views: An Error has occured'


def delete_custom_view(auth, url, name):
    """
    function takes input of auth, url, and name and issues a RESTFUL call to delete a specific of custom views from HPE
    IMC.
    :param name: string containg the name of the desired custom view

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str of creation results ( "view " + name + "created successfully"

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.groups import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_custom_view(auth.creds, auth.url, name = "L1 View")
    'View Test View deleted successfully'
    >>> delete_custom_view(auth.creds, auth.url, name = "L2 View")
    'View L2 View deleted successfully'

    """
    view_id  = get_custom_views(auth, url,name )[0]['symbolId']
    delete_custom_view_url = '/imcrs/plat/res/view/custom/'+str(view_id)
    f_url = url + delete_custom_view_url
    r = requests.delete(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 204:
            return 'View ' + name +' deleted successfully'
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' delete_custom_view: An Error has occured'


#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests
from pyhpeimc.plat.device import *


HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}




def get_vm_host_info(hostip, auth, url):
    """
    function takes hostId as input to RESTFUL call to HP IMC
    :param hostip: int or string of hostip of Hypervisor host
    :return:list of dictionatires contraining the VM Host information for the target hypervisor
    """
    hostId = get_dev_details(hostip, auth, url)['id']
    get_vm_host_info_url = "/imcrs/vrm/host?hostId=" + str(hostId)
    f_url = url + get_vm_host_info_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    # print(r.status_code)
    try:
        if r.status_code == 200:
            if len(r.text) > 0:
                return json.loads(r.text)
        elif r.status_code == 204:
            print("Device is not a supported Hypervisor")
            return "Device is not a supported Hypervisor"
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_vm_host_info: An Error has occured"

def get_vm_host_vnic(hostip, auth, url):
    """
    function takes hostId as input to RESTFUL call to HP IMC
    :param hostip: int or string of hostip of Hypervisor host
    :return:list of dictionatires contraining the VM Host Virtual NIC information for the target hypervisor
    """
    hostId = get_dev_details(hostip, auth, url)['id']
    get_vm_host_vnic_url = "/imcrs/vrm/host/vnic?hostDevId=" + str(hostId)
    f_url = url + get_vm_host_vnic_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    # print(r.status_code)
    try:
        if r.status_code == 200:
            if len(r.text) > 0:
                return json.loads(r.text)['Nic']
        elif r.status_code == 204:
            print("Device is not a supported Hypervisor")
            return "Device is not a supported Hypervisor"
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_vm_host_info: An Error has occured"

def get_host_vms(hostip, auth, url):
    """
    function takes hostId as input to RESTFUL call to HP IMC
    :param hostId: int or string of HostId of Hypervisor host
    :return: list of dictionaries containing the information on the Virtual Machines for the target hypervisor
    """
    hostId = get_dev_details(hostip, auth, url)['id']
    get_host_info_url = "/imcrs/vrm/host/vm?hostId=" + str(hostId)
    f_url = url + get_host_info_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            if len(json.loads(r.text)) > 1:
                return json.loads(r.text)['vmDevice']
            else:
                return "Device is not a supported Hypervisor"
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_host_vms: An Error has occured"



#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests


HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}




def get_cfg_template(auth, url, folder = None):
    '''
    Function takes no input and returns a list of dictionaries containing the configuration templates in the root folder
    of the icc configuration template library.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: List of Dictionaries containing folders and configuration files in the ICC library.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    #Get list of folders and files in root of ICC Configuration Library
    >>>get_cfg_template(auth.creds, auth.url)
    [{'appliedDevices': '',
  'cfgFileParent': '-1',
  'confFileDesc': 'Stores predefined configuration.',
  'confFileId': '1',
  'confFileName': 'Default Folder',
  'confFilePath': '/',
  'confFileType': '-1',
  'createAt': '2016-01-26 23:36:18',
  'createBy': '$SYSTEM',
  'defaultConfFile': '1',
  'defaultConfFileDesc': 'Default config file, cannot delete.',
  'modifyAt': '2016-01-26 23:36:18.483',
  'syncType': '0'},
 {'appliedDevices': '',
  'cfgFileParent': '-1',
  'confFileDesc': 'Folder to Store AutoDeployment Plan related configuration templates',
  'confFileId': '57',
  'confFileName': 'ADP_Configs',
  'confFilePath': '\\',
  'confFileType': '-1',
  'createAt': '2016-01-27 14:44:50',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'syncType': '0'},
 {'appliedDevices': '',
  'cfgFileParent': '-1',
  'confFileDesc': 'Juniper SNMP Strings',
  'confFileId': '62',
  'confFileName': 'JuniperSNMP.cfg',
  'confFilePath': '\\',
  'confFileType': '2',
  'confFileTypeDesc': 'Segment',
  'createAt': '2016-07-08 13:36:45',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'syncType': '0'},
 {'appliedDevices': '',
  'cfgFileParent': '-1',
  'confFileDesc': '',
  'confFileId': '61',
  'confFileName': 'SSHNAT.cfg',
  'confFilePath': '\\',
  'confFileType': '2',
  'confFileTypeDesc': 'Segment',
  'createAt': '2016-06-22 10:02:07',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'modifyAt': '2016-06-22 10:05:48',
  'syncType': '0'},
 {'appliedDevices': 'HP 5900-AF-48XGT-4QSFP+',
  'cfgFileParent': '-1',
  'confFileDesc': 'SNMP. Tempalte for V2c for Comware Device',
  'confFileId': '59',
  'confFileName': '10.20.10.10_running_20160610095951.cfg',
  'confFilePath': '\\',
  'confFileType': '2',
  'confFileTypeDesc': 'Segment',
  'createAt': '2016-06-10 13:21:32',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'modifyAt': '2016-06-10 13:24:15',
  'syncType': '0'}]

    #Get list of Folders in Files in a specific folder of the ICC Configuration Library
    >>>get_cfg_template(auth.creds, auth.url, folder='ADP_Configs')
[{'appliedDevices': 'HP 5900-AF-48XGT-4QSFP+',
  'cfgFileParent': '57',
  'confFileDesc': '5900 ADP Config with Netconf - Ansible Ready',
  'confFileId': '60',
  'confFileName': '5900ADP_w_Netconf.cfg',
  'confFilePath': '\\ADP_Configs',
  'confFileType': '1',
  'confFileTypeDesc': 'File',
  'createAt': '2016-06-21 12:01:14',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'syncType': '0'},
 {'appliedDevices': '',
  'cfgFileParent': '57',
  'confFileDesc': '5900_Initial Config for ADP process',
  'confFileId': '58',
  'confFileName': '5900_Initial.cfg',
  'confFilePath': '\\ADP_Configs',
  'confFileType': '1',
  'confFileTypeDesc': 'File',
  'createAt': '2016-01-27 14:47:59',
  'createBy': 'admin',
  'defaultConfFile': '2',
  'defaultConfFileDesc': 'Useful config file.',
  'modifyAt': '2016-01-28 14:37:21',
  'syncType': '0'}]

    '''
    if folder == None:
        get_cfg_template_url = "/imcrs/icc/confFile/list"
    else:
        folder_id = get_folder_id(folder, auth, url)
        get_cfg_template_url = "/imcrs/icc/confFile/list/"+str(folder_id)
    f_url = url + get_cfg_template_url
    r = requests.get(f_url,auth=auth, headers=HEADERS)
    #print (r.status_code)
    try:
        if r.status_code == 200:
            cfg_template_list = (json.loads(r.text))
            return cfg_template_list['confFile']

    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_cfg_template: An Error has occured"



def create_cfg_segment(filename, filecontent, description, auth, url):
    '''
    Takes a str into var filecontent which represents the entire content of a configuration segment, or partial
    configuration file. Takes a str into var description which represents the description of the configuration segment
    :param filename: str containing the name of the configuration segment.

    :param filecontent: str containing the entire contents of the configuration segment

    :param description: str contrianing the description of the configuration segment

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: If successful, Boolena of type True

    :rtype: Boolean

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> filecontent = """ snmp-agent\r\n snmp-agent community write ${ SNMP Write}\r\n snmp-agent community read
                           ${SNMP Read}\r\n snmp-agent sys-info version v2c \r\n snmp-agent target-host trap address
                           udp-domain 10.101.0.203 params securityname public\r\n snmp-agent trap enable arp \r\n
                           snmp-agent trap enable radius \r\n"""
     >>> create_cfg_segment('CW7SNMP.cfg', filecontent, 'My New Template', auth.creds, auth.url)
     True

    '''
    payload = {"confFileName": filename,
               "confFileType": "2",
               "cfgFileParent": "-1",
               "confFileDesc": description,
               "content": filecontent}
    create_cfg_segment_url = "/imcrs/icc/confFile"
    f_url = url + create_cfg_segment_url
    # creates the URL using the payload variable as the contents
    r = requests.post(f_url,data= (json.dumps(payload)), auth=auth, headers=HEADERS)
    try:
        if r.status_code == 201:
            return True
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " create_cfg_segment: An Error has occured"



def delete_cfg_template(template_name, auth, url):
    '''Uses the get_template_id() funct to gather the template_id to craft a url which is sent to the IMC server using
    a Delete Method
    :param template_name: str containing the entire contents of the configuration segment

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: If successful, Boolean of type True

    :rtype: Boolean

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_cfg_template('CW7SNMP.cfg', auth.creds, auth.url)
    True
    '''
    file_id = get_template_id(template_name, auth, url)
    delete_cfg_template_url = "/imcrs/icc/confFile/"+str(file_id)
    f_url = url + delete_cfg_template_url
    # creates the URL using the payload variable as the contents
    r = requests.delete(f_url, auth=auth, headers=HEADERS)
    #print (r.status_code)
    try:
        if r.status_code == 204:
            return True
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " delete_cfg_template: An Error has occured"

def get_folder_id(folder_name, auth, url):
    """
    Helper function takes str input of folder name and returns str numerical id of the folder.
    :param folder_name: str name of the folder

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str numerical id of the folder

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>>get_folder_id('Default Folder', auth.creds, auth.url)
    '1'
    """
    object_list = get_cfg_template(auth=auth, url=url)
    for object in object_list:
        if object['confFileName'] == folder_name:
            return object['confFileId']
    return "Folder not found"

def get_template_id(template_name, auth, url):
    """
    Helper function takes str input of folder name and returns str numerical id of the folder.
    :param folder_name: str name of the folder

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str numerical id of the folder

    :rtype: str

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>>get_template_id('CW7SNMP.cfg', auth.creds, auth.url)
    '64'
    """
    object_list = get_cfg_template(auth=auth, url=url)
    for object in object_list:
        if object['confFileName'] == template_name:
            return object['confFileId']
    return "template not found"




#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json


HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

#command creates dummy IMCAuth object. Should be overwritten in script calling these functions



"""
This section contains functions which operate at the device level
"""
def get_dev_vlans(devid, auth, url):
    """Function takes input of devID to issue RESTUL call to HP IMC
    :param devid: requires devId as the only input parameter
    :return: dictionary of existing vlans on the devices. Device must be supported in HP IMC platform VLAN manager module
    """

    # checks to see if the imc credentials are already available
    get_dev_vlans_url = "/imcrs/vlan?devId=" + str(devid) + "&start=0&size=5000&total=false"
    f_url = url + get_dev_vlans_url

    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=HEADERS)
    # r.status_code
    try:
        if r.status_code == 200:
            dev_vlans = (json.loads(r.text))
            return dev_vlans['vlan']
        elif r.status_code == 409:
            return {'vlan': 'no vlans'}
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_dev_vlans: An Error has occured'

def get_trunk_interfaces(devId, auth, url):
    """Function takes devId as input to RESTFULL call to HP IMC platform
    :param devId: output of get_dev_details
    :return: list of dictionaries containing of interfaces configured as an 802.1q trunk

    Example:
        auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")
        get_dev_asset_details("2", auth.creds, auth.url)
    """

    # checks to see if the imc credentials are already available
    get_trunk_interfaces_url = "/imcrs/vlan/trunk?devId=" + str(devId) + "&start=1&size=5000&total=false"
    f_url = url + get_trunk_interfaces_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)
    # r.status_code
    try:
        if r.status_code == 200:
            dev_trunk_interfaces = (json.loads(r.text))
        if len(dev_trunk_interfaces) == 2:
            return dev_trunk_interfaces['trunkIf']
        else:
            dev_trunk_interfaces['trunkIf'] = ["No trunk inteface"]
            return dev_trunk_interfaces['trunkIf']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' get_trunk_interfaces: An Error has occured'




def get_device_access_interfaces(devId, auth, url):
    """Function takes devId as input to RESTFUL call to HP IMC platform
    :param devId: requires deviceID as the only input parameter
    :return: list of dictionaries containing interfaces configured as access ports
    """
    # checks to see if the imc credentials are already available
    get_access_interface_vlan_url = "/imcrs/vlan/access?devId=" + str(devId) + "&start=1&size=500&total=false"
    f_url = url + get_access_interface_vlan_url
    payload = None
    # creates the URL using the payload variable as the contents
    r = requests.get(f_url, auth=auth, headers=HEADERS)
    # r.status_code
    try:
        if r.status_code == 200:
            dev_access_interfaces = (json.loads(r.text))
            if len(dev_access_interfaces) == 2:
                return dev_access_interfaces['accessIf']
            else:
                dev_access_interfaces['accessIf'] = ["No access inteface"]
                return dev_access_interfaces['accessIf']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_device_access_interfaces: An Error has occured"


def get_access_interface_vlan(ifIndex, accessinterfacelist, auth, url):
    for i in accessinterfacelist:
        if i['ifIndex'] == ifIndex:
            return i['pvid']
        else:
            return "Not an Access Port"



def create_dev_vlan(devid, vlanid, vlan_name, auth, url):
    """
    function takes devid and vlanid vlan_name of specific device and 802.1q VLAN tag and issues a RESTFUL call to add the
    specified VLAN from the target device. VLAN Name MUST be valid on target device.
    :param devid: int or str value of the target device
    :param vlanid:int or str value of target 802.1q VLAN
    :param vlan_name: str value of the target 802.1q VLAN name. MUST be valid name on target device.
    :return:HTTP Status code of 201 with no values.
    """
    create_dev_vlan_url = "/imcrs/vlan?devId=" + str(devid)
    f_url = url + create_dev_vlan_url
    payload = '''{ "vlanId": "''' + str(vlanid) + '''", "vlanName" : "''' + str(vlan_name) + '''"}'''
    r = requests.post(f_url, data=payload, auth=auth,
                      headers=HEADERS)  # creates the URL using the payload variable as the contents
    return r
    try:

        if r.status_code == 201:
            print ('Vlan Created')
            return r.status_code
        elif r.status_code == 409:
            return '''Unable to create VLAN.\nVLAN Already Exists\nDevice does not support VLAN function'''
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " create_dev_vlan: An Error has occured"


def delete_dev_vlans(devid, vlanid, auth, url):
    """
    function takes devid and vlanid of specific device and 802.1q VLAN tag and issues a RESTFUL call to remove the
    specified VLAN from the target device.
    :param devid: int or str value of the target device
    :param vlanid:
    :return:HTTP Status code of 204 with no values.
    """
    remove_dev_vlan_url = "/imcrs/vlan/delvlan?devId=" + str(devid) + "&vlanId=" + str(vlanid)
    f_url = url + remove_dev_vlan_url
    payload = None
    r = requests.delete(f_url, auth=auth,
                        headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 204:
            print ('Vlan deleted')
            return r.status_code
        elif r.status_code == 409:
            print ('Unable to delete VLAN.\nVLAN does not Exist\nDevice does not support VLAN function')
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " delete_dev_vlans: An Error has occured"




"""
This section contains functions which operate at the interface level
"""


#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import requests
import json
from requests.auth import HTTPDigestAuth


headers = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}


class IMCAuth(requests.auth.HTTPDigestAuth):
    """
    This class handles authentication against the HPE IMC API and uses the Requests API. IMCAuth derives from
    requests.auth.HTTPDigestAuth.

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> auth.password
    admin

    """

    def __init__(self, h_url,server,port,username, password):
        """
        Initializes the class. Set the h_url, server, username, and password variables.
        :param h_url: str value. Must be equal to "http://" or "https://
        :param server: str value. Must be valid IPv4 address or FQDN
        :param port: str value. Equal to listening port of IMC server. example "8080" for http or "8443" for HTTPS
        :param username: str value. Equal to username of IMC operator with privileges to access RESTUL API
        :param password: str value. Equal to valid password of username defined above
        :return:
        returns HTTPDigestauth object
        """
        super(HTTPDigestAuth,self).__init__()
        self.h_url = h_url
        self.server = server
        self.port = port
        self.username = username
        self.password = password
        self.url = self.h_url + self.server + ":" + self.port
        self.creds = requests.auth.HTTPDigestAuth(self.username, self.password)

    def get_auth(self):
        """
        This method requests an authentication object from the HPE IMC NMS and returns an HTTPDigest Auth Object
        :return:
        """
        url = self.h_url + self.server + ":" + self.port
        auth = requests.auth.HTTPDigestAuth(self.username,self.password)
        auth_url = "/imcrs"
        f_url = url + auth_url
        try:
            r = requests.get(f_url, auth=auth, headers=headers, verify=False)
            return r.status_code
    # checks for reqeusts exceptions
        except requests.exceptions.RequestException as e:
            return ("Error:\n" + str(e) + '\n\nThe IMC server address is invalid. Please try again')
            set_imc_creds()
        if r.status_code != 200:  # checks for valid IMC credentials
            return ("Error:\n" + str(e) +"Error: \n You're credentials are invalid. Please try again\n\n")
            set_imc_creds()


def test_imc_creds(auth, url):
    test_url = '/imcrs'
    f_url = url + test_url
    try:
        r = requests.get(f_url, auth=auth, headers=headers, verify=False)
        if r.status_code == 200:
            return True
        else:
            return False
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + " test_imc_creds: An Error has occured"


def set_imc_creds(h_url=None, imc_server=None, imc_port=None, imc_user=None,imc_pw=None):
    """ This function prompts user for IMC server information and credentuials and stores
    values in url and auth global variables"""
    global auth, url
    if h_url is None:
        imc_protocol = input(
        "What protocol would you like to use to connect to the IMC server: \n Press 1 for HTTP: \n Press 2 for HTTPS:")
        if imc_protocol == "1":
            h_url = 'http://'
        else:
            h_url = 'https://'
        imc_server = input("What is the ip address of the IMC server?")
        imc_port = input("What is the port number of the IMC server?")
        imc_user = input("What is the username of the IMC eAPI user?")
        imc_pw = input('''What is the password of the IMC eAPI user?''')
    url = h_url + imc_server + ":" + imc_port
    auth = requests.auth.HTTPDigestAuth(imc_user, imc_pw)
    test_url = '/imcrs'
    f_url = url + test_url
    try:
        r = requests.get(f_url, auth=auth, headers=headers, verify=False)
        print (r.status_code)
        return auth
    # checks for reqeusts exceptions
    except requests.exceptions.RequestException as e:
        print("Error:\n" + str(e))
        print("\n\nThe IMC server address is invalid. Please try again\n\n")
        set_imc_creds()
    if r.status_code != 200:  # checks for valid IMC credentials
        print("Error: \n You're credentials are invalid. Please try again\n\n")
        set_imc_creds()
    else:
        print("You've successfully access the IMC eAPI")



"""
This section contains misc helper functions as needed.
"""


def print_to_file(object):
    """
    Function takes in object of type str, list, or dict and prints out to current working directory as pyoutput.txt
    :param:  Object: object of type str, list, or dict
    :return: No return. Just prints out to file handler and save to current working directory as pyoutput.txt
    """
    with open ('pyoutput.txt', 'w') as fh:
        x = None
        if type(object) is list:
            x = json.dumps(object, indent = 4)
        if type(object) is dict:
            x = json.dumps(object, indent = 4)
        if type (object) is str:
            x = object
        fh.write(x)


#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests



HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}

def get_ac_info_all(auth, url):
    """
    function takes no input as input to RESTFUL call to HP IMC
    :return:list of dictionatires contraining the Access Controller information as known by HPE IMC Wireless Services
    Manager module.
    """
    get_ac_info_all_url = "/imcrs/wlan/acInfo/queryAcBasicInfo"
    f_url = url + get_ac_info_all_url
    payload = None
    r = requests.get(f_url, auth=auth,
                     headers=HEADERS)  # creates the URL using the payload variable as the contents
    # print(r.status_code)
    try:
        if r.status_code == 200:
            if len(r.text) > 0:
                return json.loads(r.text)['acBasicInfo']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ac_info_all: An Error has occured"


#!/usr/bin/env python3
# author: @netmanchris



# This section imports required libraries
import json
import requests
import ipaddress


HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}





def get_real_time_locate(ipAddress, auth, url):
    """
    function takes the ipAddress of a specific host and issues a RESTFUL call to get the device and interface that the
    target host is currently connected to.
    :param ipAddress: str value valid IPv4 IP address
    :return: dictionary containing hostIp, devId, deviceIP, ifDesc, ifIndex
    """
    real_time_locate_url = "/imcrs/res/access/realtimeLocate?type=2&value=" + str(ipAddress) + "&total=false"
    f_url = url + real_time_locate_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            response =  json.loads(r.text)
            if 'realtimeLocation' in response:
                real_time_locate = json.loads(r.text)['realtimeLocation']
                if type(real_time_locate) is dict:
                    real_time_locate = [real_time_locate]
                    return real_time_locate
                else:
                    return json.loads(r.text)['realtimeLocation']
            else:
                return json.loads(r.text)

    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_real_time_locate: An Error has occured"


def get_ip_mac_arp_list(devId, auth,url):
    """
    function takes devid of specific device and issues a RESTFUL call to get the IP/MAC/ARP list from the target device.
    :param devId: int or str value of the target device.
    :return: list of dictionaries containing the IP/MAC/ARP list of the target device.
    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()
    ip_mac_arp_list_url = "/imcrs/res/access/ipMacArp/" + str(devId)
    f_url = url + ip_mac_arp_list_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            macarplist = (json.loads(r.text))
            if len(macarplist) > 1:
                return macarplist['ipMacArp']
            else:
                return ['this function is unsupported']

    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_mac_arp_list: An Error has occured"


#this section deals with the IP Address Manager functions with terminal access of HPE IMC Base platform

def get_ip_scope(auth, url,scopeId=None):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE IMC server. If the
    optional scopeId parameter is included, this will automatically return only the desired scope id.
    :param scopeId: integer of the desired scope id ( optional )
    :param auth:
    :return:
    """
    if scopeId is None:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope"
    else:
        get_ip_scope_url = "/imcrs/res/access/assignedIpScope/ip?ipScopeId="+str(scopeId)

    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            return ipscopelist['assignedIpScope']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"

def get_ip_scope_detail(auth, url,scopeId=None):
    """
    function requires no inputs and returns all IP address scopes currently configured on the HPE IMC server. If the
    optional scopeId parameter is included, this will automatically return only the desired scope id.
    :param scopeId: integer of the desired scope id ( optional )
    :param auth:
    :return:
    """
    get_ip_scope_url = "/imcrs/res/access/assignedIpScope/"+str(scopeId)

    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            return ipscopelist
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"


def get_ip_scope_hosts(auth, url, scopeId):
    """
    Function requires input of scope ID and returns list of allocated IP address for the specified scope
    :param auth:
    :param url:
    :param scopeId: Interger of teh desired scope id
    :return:
    """
    get_ip_scope_url = "/imcrs/res/access/assignedIpScope/ip?size=10000&ipScopeId="+str(scopeId)
    f_url = url + get_ip_scope_url
    r = requests.get(f_url, auth=auth, headers=HEADERS)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            ipscopelist = (json.loads(r.text))
            return ipscopelist['assignedIpInfo']
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " get_ip_scope: An Error has occured"






def add_ip_scope(auth, url,startIp, endIp, name, description):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope to terminal access
    in the HPE IMC base platform
    :param startIp: str Start of IP address scope ex. '10.101.0.1'
    :param endIp: str End of IP address scope ex. '10.101.0.254'
    :param name: str Name of the owner of this IP scope  ex. 'admin'
    :param description: str description of the Ip scope
    :return:
    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()

    add_ip_scope_url = "/imcrs/res/access/assignedIpScope"
    f_url = url + add_ip_scope_url
    payload = ('''{  "startIp": "%s", "endIp": "%s","name": "%s","description": "%s" }'''
               %(str(startIp), str(endIp), str(name), str(description)))
    r = requests.post(f_url, auth=auth, headers=HEADERS, data=payload) # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print ("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def add_child_ip_scope(auth, url,startIp, endIp, name, description, scopeid):
    """
    Function takes input of four strings Start Ip, endIp, name, and description to add new Ip Scope to terminal access
    in the HPE IMC base platform
    :param startIp: str Start of IP address scope ex. '10.101.0.1'
    :param endIp: str End of IP address scope ex. '10.101.0.254'
    :param name: str Name of the owner of this IP scope  ex. 'admin'
    :param description: str description of the Ip scope
    :return:
    """
    if auth is None or url is None:  # checks to see if the imc credentials are already available
        set_imc_creds()

    add_ip_scope_url = "/imcrs/res/access/assignedIpScope/" + str(scopeid)
    f_url = url + add_ip_scope_url
    payload = ('''{  "startIp": "%s", "endIp": "%s","name": "%s","description": "%s", "parentId" : "%s"}'''
               %(str(startIp), str(endIp), str(name), str(description), str(scopeid)))
    r = requests.post(f_url, auth=auth, headers=HEADERS, data=payload) # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print ("Conflict with Current Scope")
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def add_scope_ip(ipaddress, name, description, scopeid, auth, url):
    """
    Function to add new IP address allocation to existing scope ID
    :param ipaddress:
    :param name:
    :param description:
    :param auth:
    :param url:
    :return:
    """
    new_ip = { "ip": ipaddress,
      "name": name,
      "description": description}
    add_scope_ip_url = '/imcrs/res/access/assignedIpScope/ip?ipScopeId='+str(scopeid)
    f_url = url + add_scope_ip_url
    payload = json.dumps(new_ip)
    r = requests.post(f_url, auth=auth, headers=HEADERS,
                      data=payload)  # creates the URL using the payload variable as the contents
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"

def remove_scope_ip(hostid, auth, url):
    """
    Function to add new IP address allocation to existing scope ID
    :param ipaddress:
    :param name:
    :param description:
    :param auth:
    :param url:
    :return:
    """
    add_scope_ip_url = '/imcrs/res/access/assignedIpScope/ip/'+str(hostid)
    f_url = url + add_scope_ip_url

    r = requests.delete(f_url, auth=auth, headers=HEADERS,
                      )
    try:
        if r.status_code == 200:
            print("IP Scope Successfully Created")
            return r.status_code
        elif r.status_code == 409:
            print("IP Scope Already Exists")
            return r.status_code
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + " add_ip_scope: An Error has occured"



def get_scope_id(network_address, auth, url):
    netaddr = ipaddress.ip_network(network_address)
    scopes = get_ip_scope(auth, url)
    for i in scopes:
        if int(i['id']) > 0:
            if ipaddress.ip_address(i['startIp']) in netaddr and ipaddress.ip_address(i['endIp']) in netaddr:
                return i['id']
            if "assignedIpScope" in i:
                for child in i['assignedIpScope']:
                    if ipaddress.ip_address(child['startIp']) in netaddr and ipaddress.ip_address(child['endIp']) in netaddr:

                        return child['id']









    #Add host to IP scope
    #http://10.101.0.203:8080/imcrs/res/access/assignedIpScope/ip?ipScopeId=1
    '''{
      "ip": "10.101.0.1",
      "name": "Cisco2811.lab.local",
      "description": "Cisco 2811",
      "parentId": "18"
    }'''


#!/usr/bin/env python3
# author: @netmanchris

# This section imports required libraries
import requests
import json



HEADERS = {'Accept': 'application/json', 'Content-Type':
    'application/json', 'Accept-encoding': 'application/json'}


"""
This section deals with HPE IMC Operator Functions
"""

def create_operator(operator, auth, url,headers=HEADERS):
    """
    Function takes input of dictionary operator with the following keys
    operator = { "fullName" : ""   ,
             "sessionTimeout" : "",
             "password" :  "",
             "operatorGroupId" : "",
             "name" : "",
             "desc" : "",
             "defaultAcl" : "",
             "authType"  : ""}
    converts to json and issues a HTTP POST request to the HPE IMC Restful API

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param operator: dictionary with the required operator key-value pairs as defined above.

    :param headers: json formated string. default values set in module

    :return:

    :rtype:


    >>> import json

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>>  operator = '''{ "fullName" : "test administrator"   ,
             "sessionTimeout" : "30",
             "password" :  "password",
             "operatorGroupId" : "1",
             "name" : "testadmin",
             "desc" : "test admin account",
             "defaultAcl" : "",
             "authType"  : "0"}'''

    >>> operator = json.loads(operator)

    >>> create_operator(operator, auth.creds, auth.url)
    Operator Successfully Created

    """
    create_operator_url = '/imcrs/plat/operator'
    f_url = url + create_operator_url
    print (f_url)
    payload = json.dumps(operator, indent=4)
    # creates the URL using the payload variable as the contents
    r = requests.post(f_url, data=payload, auth=auth, headers=headers)
    try:
        if r.status_code == 409:
            print("Operator Already Exists")
            return r.status_code
        elif r.status_code == 201:
            print("Operator Successfully Created")
            return r.status_code
    except requests.exceptions.RequestException as e:
            return "Error:\n" + str(e) + ' create_operator: An Error has occured'

def set_operator_password(operator, password, auth, url,headers=HEADERS):
    """
    Function to set the password of an existing operator

    :param operator: str Name of the operator account

    :param password: str New password

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param headers: json formated string. default values set in module

    :return: int of 204 if successfull,

    :rtype: int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> set_operator_password('testadmin', 'newpassword', auth.creds, auth.url)
    Operator:testadmin password was successfully changed
    204

       """
    if operator == None:
        operator = input(
            '''\n What is the username you wish to change the password?''')
    oper_id = ''
    plat_oper_list = get_plat_operator(auth, url)
    for i in plat_oper_list:
        if i['name'] == operator:
            oper_id = i['id']
            authType = i['authType']
    if oper_id == '':
        return("\n User does not exist")
    change_pw_url = "/imcrs/plat/operator/"
    f_url = url + change_pw_url + oper_id
    if password is None:
        password = input(
        '''\n ============ Please input the operators new password:\n ============  ''')
    payload = json.dumps({'password': password , 'authType': authType})
    r = requests.put(f_url, data=payload, auth=auth, headers=headers)
    try:
        if r.status_code == 204:
            print("\n Operator:" + operator +
                  " password was successfully changed")
            return r.status_code
    except requests.exceptions.RequestException as e:
        return "Error:\n" + str(e) + ' set_operator_password: An Error has occured'

def get_plat_operator(auth, url,headers=HEADERS):
    '''
    Funtion takes no inputs and returns a list of dictionaties of all of the operators currently configured on the HPE
    IMC system

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element represents one operator

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> get_plat_operator(auth.creds, auth.url)

    '''
    get_operator_url = '/imcrs/plat/operator?start=0&size=1000&orderBy=id&desc=false&total=false'
    f_url = url + get_operator_url
    try:
        r = requests.get(f_url, auth=auth, headers=headers)
        plat_oper_list = json.loads(r.text)['operator']
        if type(plat_oper_list) is dict:
            oper_list = []
            oper_list.append(plat_oper_list)
            return oper_list
        return plat_oper_list
    except requests.exceptions.RequestException as e:
        print ("Error:\n" + str(e) + ' get_plat_operator: An Error has occured')
        return "Error:\n" + str(e) + ' get_plat_operator: An Error has occured'

def delete_plat_operator(operator,auth, url, headers=HEADERS):
    """
    Function to set the password of an existing operator
    :param operator: str Name of the operator account

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param headers: json formated string. default values set in module

    :return: int of 204 if successfull

    :rtype: int

    >>> delete_plat_operator('testadmin', auth.creds, auth.url)
    Operator: testadmin was successfully deleted
    204
    """
    #oper_id = None
    plat_oper_list = get_plat_operator(auth, url)
    for i in plat_oper_list:
        if operator == i['name']:
            oper_id = i['id']
    if oper_id == None:
        return("\n User does not exist")
    delete_plat_operator_url = "/imcrs/plat/operator/"
    f_url = url + delete_plat_operator_url + str(oper_id)
    r = requests.delete(f_url, auth=auth, headers=headers)
    try:
        if r.status_code == 204:
            print("\n Operator: " + operator +
                  " was successfully deleted")
            return r.status_code
    except requests.exceptions.RequestException as e:
        print ("Error:\n" + str(e) + ' delete_plat_operator: An Error has occured')
        return "Error:\n" + str(e) + ' delete_plat_operator: An Error has occured'


